@@ -252,8 +252,10 @@ class FICSPlayer (GObject.GObject):
         else: return True
     
     def isObservable (self):
-        return self.status in (IC_STATUS_PLAYING, IC_STATUS_EXAMINING) and \
-               self.game is not None and not self.game.private and self.game.supported
+        return self.status == IC_STATUS_EXAMINING \
+                or \
+              (self.status == IC_STATUS_PLAYING and \
+                self.game is not None and not self.game.private and self.game.supported)
         
     def isGuest (self):
         return TYPE_UNREGISTERED in self.titles
@@ -423,13 +425,13 @@ class FICSPlayers (GObject.GObject):
     
     def online_changed (self, player, prop):
         if player.online:
-            GLib.idle_add(self.emit, "FICSPlayerEntered", player, priority=GLib.PRIORITY_LOW)
+            GLib.idle_add(self.emit, "FICSPlayerEntered", [player,], priority=GLib.PRIORITY_LOW)
     
     # This method is a temporary hack until ChatWindow/ChatManager are
     # converted to use FICSPlayer references rather than player's names
     def get_online_playernames (self):
         names = []
-        players = self.values()
+        players = list(self.values())
         for player in players:
             if player.online:
                 names.append(player.name)
@@ -500,8 +502,9 @@ class FICSMatch (GObject.GObject):
 
     @property
     def sortable_time (self):
-        gainminutes = (self.inc*60)-1 if self.inc != None and self.inc > 0 else 0
-        return self.minutes*60 + gainminutes
+        #http://www.freechess.org/Help/HelpFiles/etime.html
+        etime = self.minutes + int(round(self.inc*2./3.))
+        return etime
     
 def get_soughtmatch_tooltip_text (sought):
     text = "%s" % sought.player.name
@@ -739,7 +742,7 @@ class FICSBoard (object):
 class FICSGame (FICSMatch):
     def __init__ (self, wplayer, bplayer, gameno=None,
                   game_type=None, rated=False, minutes=None, inc=None, result=None,
-                  reason=None, board=None, private=False):
+                  reason=None, board=None, private=False, relation=None):
         assert isinstance(wplayer, FICSPlayer), wplayer
         assert isinstance(bplayer, FICSPlayer), bplayer
         assert gameno is None or isinstance(gameno, int), gameno
@@ -755,6 +758,7 @@ class FICSGame (FICSMatch):
         self.reason = reason
         self.board = board
         self.private = private
+        self.relation = relation
     
     def __hash__ (self):
         return hash(":".join((self.wplayer.name[0:10].lower(),
@@ -939,7 +943,7 @@ class FICSGames (GObject.GObject):
         elif create:
             self[game] = game
             if emit:
-                self.emit("FICSGameCreated", game)
+                self.emit("FICSGameCreated", [game,])
         else:
             raise KeyError
         return game
