@@ -169,6 +169,7 @@ class BoardManager (GObject.GObject):
         'obsGameEnded'        : (GObject.SignalFlags.RUN_FIRST, None, (object,)),
         'curGameEnded'        : (GObject.SignalFlags.RUN_FIRST, None, (object,)),
         'obsGameUnobserved'   : (GObject.SignalFlags.RUN_FIRST, None, (object,)),
+        'madeExaminer'        : (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         'gamePaused'          : (GObject.SignalFlags.RUN_FIRST, None, (int, bool)),
         'tooManySeeks'        : (GObject.SignalFlags.RUN_FIRST, None, ()),
         'matchDeclined'       : (GObject.SignalFlags.RUN_FIRST, None, (object,)),
@@ -228,13 +229,19 @@ class BoardManager (GObject.GObject):
 
         self.connection.expect_n_lines (self.onObserveGameCreated,
             "You are now observing game \d+\.",
-            "Game (\d+): %s %s %s %s %s [\w/]+ \d+ \d+"
-            % (names, ratings, names, ratings, ratedexp))
+            "Game (\d+): %s %s %s %s %s ([\w/]+) (\d+) (\d+)"
+            % (names, ratings, names, ratings, ratedexp),
+            '',
+            "<12> (.+)")
+
         self.connection.expect_n_lines (self.onFollowingPlayer,
             "You will now be following %s's games\." % names,
             "You are now observing game \d+\.",
-            "Game (\d+): %s %s %s %s %s [\w/]+ \d+ \d+"
-            % (names, ratings, names, ratings, ratedexp))
+            "Game (\d+): %s %s %s %s %s ([\w/]+) (\d+) (\d+)"
+            % (names, ratings, names, ratings, ratedexp),
+            '',
+            "<12> (.+)")
+
         self.connection.expect_fromto (self.onObserveGameMovesReceived,
             "Movelist for game (\d+):", "{Still in progress} \*")
         self.connection.expect_line (self.onGamePause,
@@ -242,6 +249,9 @@ class BoardManager (GObject.GObject):
         self.connection.expect_line (self.onUnobserveGame,
                 "Removing game (\d+) from observation list\.")
         
+        self.connection.expect_line (self.made_examiner,
+            "%s has made you an examiner of game (\d+)\." % names)
+        
         self.queuedEmits = {}
         self.gamemodelStartedEvents = {}
         self.theGameImPlaying = None
@@ -652,7 +662,7 @@ class BoardManager (GObject.GObject):
         
         pgnHead = [
             ("Event", "FICS %s %s game" % (rated.lower(), game_type.fics_name)),
-            ("Site", "FICS"),
+            ("Site", "freechess.org"),
             ("White", wname),
             ("Black", bname),
             ("TimeControl", "%d+%d" % (minutes * 60, increment)),
@@ -720,27 +730,60 @@ class BoardManager (GObject.GObject):
     def onObserveGameCreated (self, matchlist):
         log.debug("'%s'" % (matchlist[1].string),
             extra={"task": (self.connection.username, "BM.onObserveGameCreated")})
-        gameno, wname, wrating, bname, brating, rated = matchlist[1].groups()
+        gameno, wname, wrating, bname, brating, rated, gametype, min, inc = matchlist[1].groups()
         wplayer = self.connection.players.get(FICSPlayer(wname))
         bplayer = self.connection.players.get(FICSPlayer(bname))
         game = FICSGame(wplayer, bplayer, gameno=int(gameno))
-        game = self.connection.games.get(game, emit=False)
-        
-        if not game.supported:
-            log.warning("Trying to follow an unsupported type game %s" % game.game_type)
-            return
 
-        if game.gameno in self.gamemodelStartedEvents:
-            log.warning("%s already in gamemodelstartedevents" % game.gameno)
-            return
+        style12 = matchlist[-1].groups()[0]
+        gameno = int(gameno)
+        wrating = self.parseRating(wrating)
+        brating = self.parseRating(brating)
+        game_type = GAME_TYPES[gametype]
         
-        self.gamesImObserving[game] = None
-        self.queuedStyle12s[game.gameno] = []
-        self.queuedEmits[game.gameno] = []
-        self.gamemodelStartedEvents[game.gameno] = threading.Event()
+        castleSigns = self.generateCastleSigns(style12, game_type)
+        self.castleSigns[gameno] = castleSigns
+        gameno, relation, curcol, ply, wname, bname, wms, bms, gain, lastmove, fen = \
+                self.parseStyle12(style12, castleSigns)
 
-        # FICS doesn't send the move list after 'observe' and 'follow' commands
-        self.connection.client.run_command("moves %d" % game.gameno)
+        if relation == IC_POS_OBSERVING_EXAMINATION:
+            pgnHead = [
+                ("Event", "FICS %s %s game" % (rated, game_type.fics_name)),
+                ("Site", "freechess.org"),
+                ("White", wname),
+                ("Black", bname),
+                ("Result", "*"),
+                ("SetUp", "1"),
+                ("FEN", fen)
+                ]
+            pgn = "\n".join(['[%s "%s"]' % line for line in pgnHead]) + "\n*\n"
+            game = FICSGame(wplayer, bplayer, gameno=gameno, rated=rated=="rated",
+                game_type=game_type, minutes=int(min), inc=int(inc),
+                board=FICSBoard(wms, bms, pgn=pgn), relation=relation)
+            game = self.connection.games.get(game)
+            self.gamesImObserving[game] = None
+
+            self.gamemodelStartedEvents[game.gameno] = threading.Event()
+            self.emit("obsGameCreated", game)
+            self.gamemodelStartedEvents[game.gameno].wait()
+        else:
+            game = self.connection.games.get(game, emit=False)
+            
+            if not game.supported:
+                log.warning("Trying to follow an unsupported type game %s" % game.game_type)
+                return
+
+            if game.gameno in self.gamemodelStartedEvents:
+                log.warning("%s already in gamemodelstartedevents" % game.gameno)
+                return
+            
+            self.gamesImObserving[game] = None
+            self.queuedStyle12s[game.gameno] = []
+            self.queuedEmits[game.gameno] = []
+            self.gamemodelStartedEvents[game.gameno] = threading.Event()
+
+            # FICS doesn't send the move list after 'observe' and 'follow' commands
+            self.connection.client.run_command("moves %d" % game.gameno)
     onObserveGameCreated.BLKCMD = BLKCMD_OBSERVE
 
     def onFollowingPlayer (self, matchlist):
@@ -835,7 +878,16 @@ class BoardManager (GObject.GObject):
         player = self.connection.players.get(FICSPlayer(player))
         self.emit("player_on_noplay", player)
     player_on_noplay.BLKCMD = BLKCMD_MATCH
-        
+
+    def made_examiner (self, match):
+        """ Changing from observer to examiner """
+        player, gameno = match.groups()
+        gameno = int(gameno)
+        try:
+            game = self.connection.games.get_game_by_gameno(gameno)
+        except KeyError: return
+        self.emit("madeExaminer", gameno)
+
     ############################################################################
     #   Interacting                                                            #
     ############################################################################
@@ -852,8 +904,14 @@ class BoardManager (GObject.GObject):
     def callflag (self):
         self.connection.client.run_command("flag")
     
-    def observe (self, game):
-        self.connection.client.run_command("observe %d" % game.gameno)
+    def observe (self, game, player=None):
+        if game is not None:
+            self.connection.client.run_command("observe %d" % game.gameno)
+        elif player is not None:
+            self.connection.client.run_command("observe %s" % player.name)
+
+    def unexamine (self):
+        self.connection.client.run_command("unexamine")
 
     def unobserve (self, game):
         if game.gameno is not None:
